/*
 * PmodGYRO.c
 *
 *  Created on: Nov 22, 2017
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"

#include "PmodGYRO.h"

SPI_HandleTypeDef pmodGyroSpi;

uint8_t readRegister(uint8_t address)
{
  // Write the register address with the Read flag set first.
  uint8_t txbuf[2] = {address | 0x80, 0x00};
  uint8_t rxbuf[2] = {0x00};

  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
  HAL_SPI_TransmitReceive(&pmodGyroSpi, txbuf, rxbuf, 2, 100);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);

  return rxbuf[1];
}

void writeRegister(uint8_t address, uint8_t data)
{
  // Write the register address and the given data.
  uint8_t txbuf[2] = {address, data};
  uint8_t rxbuf[2] = {0x00};

  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
  HAL_SPI_TransmitReceive(&pmodGyroSpi, txbuf, rxbuf, 2, 100);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
}


void PmodGYRO_Config(void)
{
  // Configure the SPI connected to the Pmod module.
  pmodGyroSpi.Instance = SPI1;
  pmodGyroSpi.Init.Mode = SPI_MODE_MASTER;
  pmodGyroSpi.Init.Direction = SPI_DIRECTION_2LINES;
  pmodGyroSpi.Init.DataSize = SPI_DATASIZE_8BIT;
  pmodGyroSpi.Init.CLKPolarity = SPI_POLARITY_HIGH;
  pmodGyroSpi.Init.CLKPhase = SPI_PHASE_2EDGE;
  pmodGyroSpi.Init.NSS = SPI_NSS_SOFT;
  pmodGyroSpi.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_128;
  pmodGyroSpi.Init.FirstBit = SPI_FIRSTBIT_MSB;
  pmodGyroSpi.Init.TIMode = SPI_TIMODE_DISABLE;
  pmodGyroSpi.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  pmodGyroSpi.Init.NSSPMode = SPI_NSS_PULSE_DISABLE;

  HAL_SPI_Init(&pmodGyroSpi);

  // Initialization of PmodGYRO.
  writeRegister(0x20, 0x0C); // Enable z axis, normal mode, ODR 95Hz, BW 12.5Hz.
  writeRegister(0x36, 0x10); // Set the MSB of the Z axis interrupt threshold (4096).
  writeRegister(0x37, 0x00); // Set the LSB of the Z axis interrupt threshold (4096).
  writeRegister(0x30, 0x20); // Set interrupt on Z axis over threshold.
  writeRegister(0x22, 0x80); // Enable interrupt on INT1 pin.

  // Configure the INT1 (PD2) interrupt line with the lower priority than SysTick.
  __HAL_RCC_GPIOE_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pin = GPIO_PIN_12;
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 0x0F, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);
}

void HAL_SPI_MspInit(SPI_HandleTypeDef *hspi)
{
  // Initialize GPIO used by the SPI1 peripheral. The CS is configured by the software.
  __HAL_RCC_SPI1_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOE_CLK_ENABLE();
  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF5_SPI2;
  GPIO_InitStruct.Pin = GPIO_PIN_1 | GPIO_PIN_7;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = GPIO_PIN_14;
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Pin = GPIO_PIN_0;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
}

uint8_t PmodGYRO_ReadInterruptFlags(void)
{
  return readRegister(0x31); // Read the INT1_SRC register.
}

